@@ -27,10 +27,10 @@
 #define snprintf _snprintf
 #endif
 
-#include <libdivecomputer/hw_ostc.h>
-#include <libdivecomputer/hw_ostc3.h>
 #include "libdivecomputer/units.h"
 
+#include "hw_ostc.h"
+#include "hw_ostc3.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -95,6 +95,7 @@ typedef struct hw_ostc_layout_t {
 	unsigned int deco_info1;
 	unsigned int deco_info2;
 	unsigned int decomode;
+        unsigned int battery_percentage;
 } hw_ostc_layout_t;
 
 typedef struct hw_ostc_gasmix_t {
@@ -115,6 +116,7 @@ typedef struct hw_ostc_parser_t {
 	unsigned int ngasmixes;
 	unsigned int nfixed;
 	unsigned int initial;
+	unsigned int initial_setpoint;
 	hw_ostc_gasmix_t gasmix[NGASMIXES];
 } hw_ostc_parser_t;
 
@@ -148,6 +150,7 @@ static const hw_ostc_layout_t hw_ostc_layout_ostc = {
 	49, /* deco_info1 */
 	50, /* deco_info1 */
 	51, /* decomode */
+        0,  /* battery percentage TBD */
 };
 
 static const hw_ostc_layout_t hw_ostc_layout_frog = {
@@ -165,6 +168,7 @@ static const hw_ostc_layout_t hw_ostc_layout_frog = {
 	49, /* deco_info1 */
 	50, /* deco_info2 */
 	51, /* decomode */
+        0,  /* battery percentage TBD */
 };
 
 static const hw_ostc_layout_t hw_ostc_layout_ostc3 = {
@@ -182,6 +186,7 @@ static const hw_ostc_layout_t hw_ostc_layout_ostc3 = {
 	77, /* deco_info1 */
 	78, /* deco_info2 */
 	79, /* decomode */
+        59,  /* battery percentage */
 };
 
 static unsigned int
@@ -253,8 +258,10 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 		return DC_STATUS_DATAFORMAT;
 	}
 
-	// Get all the gas mixes, and the index of the inital mix.
+	// Get all the gas mixes, the index of the inital mix,
+	// and the initial setpoint (used in the fixed setpoint mode).
 	unsigned int initial = UNDEFINED;
+	unsigned int initial_setpoint = UNDEFINED;
 	unsigned int ngasmixes = 0;
 	hw_ostc_gasmix_t gasmix[NGASMIXES] = {{0}};
 	if (version == 0x22) {
@@ -274,6 +281,10 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 				initial = i + 1; /* One based index! */
 			}
 		}
+		// The first fixed setpoint is the initial setpoint in CCR mode.
+                if (data[82] == OSTC3_CC) {
+			initial_setpoint = data[60];
+		}
 	} else {
 		ngasmixes = 5;
 		initial = data[31];
@@ -299,6 +310,7 @@ hw_ostc_parser_cache (hw_ostc_parser_t *parser)
 	parser->ngasmixes = ngasmixes;
 	parser->nfixed = ngasmixes;
 	parser->initial = initial;
+	parser->initial_setpoint = initial_setpoint;
 	for (unsigned int i = 0; i < ngasmixes; ++i) {
 		parser->gasmix[i] = gasmix[i];
 	}
@@ -332,6 +344,7 @@ hw_ostc_parser_create_internal (dc_parser_t **out, dc_context_t *context, unsign
 	parser->ngasmixes = 0;
 	parser->nfixed = 0;
 	parser->initial = 0;
+	parser->initial_setpoint = 0;
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
@@ -369,6 +382,7 @@ hw_ostc_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsig
 	parser->ngasmixes = 0;
 	parser->nfixed = 0;
 	parser->initial = 0;
+	parser->initial_setpoint = 0;
 	for (unsigned int i = 0; i < NGASMIXES; ++i) {
 		parser->gasmix[i].oxygen = 0;
 		parser->gasmix[i].helium = 0;
@@ -418,6 +432,7 @@ hw_ostc_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 	dt.hour   = p[3];
 	dt.minute = p[4];
 	dt.second = 0;
+	dt.timezone = DC_TIMEZONE_NONE;
 
 	if (version == 0x24) {
 		if (datetime)
@@ -429,14 +444,16 @@ hw_ostc_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 
 		ticks -= divetime;
 
-		if (!dc_datetime_localtime (datetime, ticks))
+		if (!dc_datetime_gmtime (datetime, ticks))
 			return DC_STATUS_DATAFORMAT;
+
+		datetime->timezone = DC_TIMEZONE_NONE;
 	}
 
 	return DC_STATUS_SUCCESS;
 }
 
-#define BUFLEN 16
+#define BUFLEN 32
 
 static dc_status_t
 hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
@@ -560,16 +577,28 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 			break;
 		case DC_FIELD_STRING:
 			switch(flags) {
-			case 0: /* battery */
+                        case 0: /* serial */
+                                string->desc = "Serial";
+                                snprintf(buf, BUFLEN, "%u", parser->serial);
+                                break;
+                        case 1: /* battery */
 				string->desc = "Battery at end";
-				snprintf(buf, BUFLEN, "%.2f", array_uint16_le (data + layout->battery) / 1000.0);
-				break;
-			case 1: /* desat */
-				string->desc = "Desat time";
+                                unsigned int percentage = (unsigned int) data[layout->battery_percentage];
+                                if (percentage != 0xFF && (version == 0x23 || version == 0x24)) {
+                                    percentage = percentage>100? 100: percentage;
+                                    snprintf(buf, BUFLEN, "%.2fV, %u%% remaining",
+                                             array_uint16_le (data + layout->battery) / 1000.0,
+                                             percentage);
+                                } else {
+                                    snprintf(buf, BUFLEN, "%.2fV", array_uint16_le (data + layout->battery) / 1000.0);
+                                }
+                                break;
+                        case 2: /* desat */
+                                string->desc = "Desat time";
 				snprintf(buf, BUFLEN, "%0u:%02u", array_uint16_le (data + layout->desat) / 60,
 						 array_uint16_le (data + layout->desat) % 60);
 				break;
-			case 2: /* firmware */
+                        case 3: /* firmware */
 				string->desc = "FW Version";
 				/* OSTC4 stores firmware as XXXX XYYY YYZZ ZZZB, -> X.Y.Z beta? */
 				if (parser->model == OSTC4) {
@@ -585,11 +614,8 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 					snprintf(buf, BUFLEN, "%0u.%02u", data[layout->firmware], data[layout->firmware + 1]);
 				}
 				break;
-			case 3: /* serial */
-				string->desc = "Serial";
-				snprintf(buf, BUFLEN, "%u", parser->serial);
-				break;
-			case 4: /* Deco model */
+
+                        case 4: /* Deco model */
 				string->desc = "Deco model";
 				if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16) ||
@@ -604,7 +630,7 @@ hw_ostc_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned
 				else
 					return DC_STATUS_DATAFORMAT;
 				break;
-			case 5: /* Deco model info */
+                        case 5: /* Deco model info */
 				string->desc = "Deco model info";
 				if (((version == 0x23 || version == 0x24) && data[layout->decomode] == OSTC3_ZHL16) ||
 						(version == 0x22 && data[layout->decomode] == FROG_ZHL16) ||
@@ -743,6 +769,12 @@ hw_ostc_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t call
 			if (callback) callback (DC_SAMPLE_GASMIX, sample, userdata);
 		}
 
+		// Initial setpoint (mbar).
+		if (time == samplerate && parser->initial_setpoint != UNDEFINED) {
+			sample.setpoint = parser->initial_setpoint / 100.0;
+			if (callback) callback (DC_SAMPLE_SETPOINT, sample, userdata);
+		}
+
 		// Depth (mbar).
 		unsigned int depth = array_uint16_le (data + offset);
 		sample.depth = (depth * BAR / 1000.0) / hydrostatic;
