@@ -23,8 +23,7 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
-#include <libdivecomputer/mares_iconhd.h>
-
+#include "mares_iconhd.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "serial.h"
@@ -43,6 +42,7 @@
 #define PUCKPRO   0x18
 #define NEMOWIDE2 0x19
 #define PUCK2     0x1F
+#define QUAD      0x29
 
 #define ACK 0xAA
 #define EOF 0xEA
@@ -87,6 +87,7 @@ static const dc_device_vtable_t mares_iconhd_device_vtable = {
 	NULL, /* write */
 	mares_iconhd_device_dump, /* dump */
 	mares_iconhd_device_foreach, /* foreach */
+	NULL, /* timesync */
 	mares_iconhd_device_close /* close */
 };
 
@@ -126,6 +127,7 @@ mares_iconhd_get_model (mares_iconhd_device_t *device)
 		{"Puck Pro",    PUCKPRO},
 		{"Nemo Wide 2", NEMOWIDE2},
 		{"Puck 2",      PUCK2},
+		{"Quad",        QUAD},
 	};
 
 	// Check the product name in the version packet against the list
@@ -210,7 +212,7 @@ mares_iconhd_transfer (mares_iconhd_device_t *device,
 
 
 dc_status_t
-mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	mares_iconhd_device_t *device = NULL;
@@ -293,6 +295,7 @@ mares_iconhd_device_open (dc_device_t **out, dc_context_t *context, const char *
 	case NEMOWIDE2:
 	case SMART:
 	case SMARTAPNEA:
+	case QUAD:
 		device->layout = &mares_nemowide2_layout;
 		device->packetsize = 256;
 		break;
