@@ -22,21 +22,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <zlib.h>	/* For crc32() */
 
-#include <libdivecomputer/suunto_eonsteel.h>
-
+#include "suunto_eonsteel.h"
 #include "context-private.h"
 #include "device-private.h"
 #include "array.h"
 #include "usbhid.h"
-
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
+#include "platform.h"
 
 typedef struct suunto_eonsteel_device_t {
 	dc_device_t base;
-	dc_usbhid_t *usbhid;
 	unsigned int magic;
 	unsigned short seq;
 	unsigned char version[0x30];
@@ -55,23 +51,29 @@ struct directory_entry {
 };
 
 // EON Steel command numbers and other magic field values
-#define INIT_CMD   0x00
-#define INIT_MAGIC 0x0001
-#define INIT_SEQ   0
+#define CMD_INIT	0x0000
+#define INIT_MAGIC	0x0001
+#define INIT_SEQ	0
 
-#define READ_STRING_CMD 0x0411
+#define CMD_READ_STRING	0x0411
 
-#define FILE_LOOKUP_CMD 0x0010
-#define FILE_READ_CMD   0x0110
-#define FILE_STAT_CMD   0x0710
-#define FILE_CLOSE_CMD  0x0510
+#define CMD_FILE_OPEN	0x0010
+#define CMD_FILE_READ	0x0110
+#define CMD_FILE_STAT	0x0710
+#define CMD_FILE_CLOSE	0x0510
 
-#define DIR_LOOKUP_CMD 0x0810
-#define READDIR_CMD    0x0910
-#define DIR_CLOSE_CMD  0x0a10
+#define CMD_DIR_OPEN	0x0810
+#define CMD_DIR_READDIR	0x0910
+#define CMD_DIR_CLOSE	0x0a10
+
+#define CMD_SET_TIME	0x0003
+#define CMD_GET_TIME	0x0103
+#define CMD_SET_DATE	0x0203
+#define CMD_GET_DATE	0x0303
 
 static dc_status_t suunto_eonsteel_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
 static dc_status_t suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
+static dc_status_t suunto_eonsteel_timesync(dc_device_t *abstract, const dc_datetime_t *datetime);
 static dc_status_t suunto_eonsteel_device_close(dc_device_t *abstract);
 
 static const dc_device_vtable_t suunto_eonsteel_device_vtable = {
@@ -82,6 +84,7 @@ static const dc_device_vtable_t suunto_eonsteel_device_vtable = {
 	NULL, /* write */
 	NULL, /* dump */
 	suunto_eonsteel_device_foreach, /* foreach */
+	suunto_eonsteel_timesync, /* timesync */
 	suunto_eonsteel_device_close /* close */
 };
 
@@ -127,20 +130,20 @@ static void put_le32(unsigned int val, unsigned char *p)
  * The maximum payload is 62 bytes.
  */
 #define PACKET_SIZE 64
-static int receive_packet(suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
+static int receive_usbhid_packet(dc_custom_io_t *io, suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
 {
 	unsigned char buf[64];
 	dc_status_t rc = DC_STATUS_SUCCESS;
 	size_t transferred = 0;
 	int len;
 
-	rc = dc_usbhid_read(eon->usbhid, buf, PACKET_SIZE, &transferred);
+	rc = io->packet_read(io, buf, PACKET_SIZE, &transferred);
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "read interrupt transfer failed");
 		return -1;
 	}
 	if (transferred != PACKET_SIZE) {
-		ERROR(eon->base.context, "incomplete read interrupt transfer (got %zu, expected %d)", transferred, PACKET_SIZE);
+		ERROR(eon->base.context, "incomplete read interrupt transfer (got " DC_PRINTF_SIZE ", expected %d)", transferred, PACKET_SIZE);
 		return -1;
 	}
 	if (buf[0] != 0x3f) {
@@ -161,6 +164,159 @@ static int receive_packet(suunto_eonsteel_device_t *eon, unsigned char *buffer,
 	return len;
 }
 
+static int fill_ble_buffer(dc_custom_io_t *io, suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
+{
+	int state = 0;
+	int bytes = 0;
+	unsigned int crc;
+
+	for (;;) {
+		unsigned char packet[32];
+		dc_status_t rc = DC_STATUS_SUCCESS;
+		size_t transferred = 0;
+		int i;
+
+		rc = io->packet_read(io, packet, sizeof(packet), &transferred);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR(eon->base.context, "BLE GATT read transfer failed");
+			return -1;
+		}
+		for (i = 0; i < transferred; i++) {
+			unsigned char c = packet[i];
+
+			if (c == 0x7e) {
+				if (state == 1)
+					goto done;
+				if (state == 2) {
+					ERROR(eon->base.context, "BLE GATT stream has escaped 7e character");
+					return -1;
+				}
+				/* Initial 7e character - good */
+				state = 1;
+				continue;
+			}
+
+			if (!state) {
+				ERROR(eon->base.context, "BLE GATT stream did not start with 7e");
+				return -1;
+			}
+
+			if (c == 0x7d) {
+				if (state == 2) {
+					ERROR(eon->base.context, "BLE GATT stream has escaped 7d character");
+					return -1;
+				}
+				state = 2;
+				continue;
+			}
+
+			if (state == 2) {
+				c ^= 0x20;
+				state = 1;
+			}
+			if (bytes < size)
+				buffer[bytes] = c;
+			bytes++;
+		}
+	}
+done:
+	if (bytes < 4) {
+		ERROR(eon->base.context, "did not receive BLE CRC32 data");
+		return -1;
+	}
+	if (bytes > size) {
+		ERROR(eon->base.context, "BLE GATT stream too long (%d bytes, buffer is %d)", bytes, size);
+		return -1;
+	}
+
+	/* Remove and check CRC */
+	bytes -= 4;
+	crc = crc32(0, buffer, bytes);
+	if (crc != array_uint32_le(buffer + bytes)) {
+		ERROR(eon->base.context, "incorrect BLE CRC32 data");
+		return -1;
+	}
+	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "rcv", buffer, bytes);
+	return bytes;
+}
+
+#define HDRSIZE 12
+#define MAXDATA 2048
+#define CRCSIZE 4
+
+static struct {
+	unsigned int len, offset;
+	unsigned char buffer[HDRSIZE + MAXDATA + CRCSIZE];
+} ble_data;
+
+static void fill_ble_data(dc_custom_io_t *io, suunto_eonsteel_device_t *eon)
+{
+	int received;
+
+	received = fill_ble_buffer(io, eon, ble_data.buffer, sizeof(ble_data.buffer));
+	if (received < 0)
+		received = 0;
+	ble_data.offset = 0;
+	ble_data.len = received;
+}
+
+static int receive_ble_packet(dc_custom_io_t *io, suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
+{
+	int maxsize;
+
+	if (ble_data.offset >= ble_data.len)
+		return 0;
+	maxsize = ble_data.len - ble_data.offset;
+	if (size > maxsize)
+		size = maxsize;
+	memcpy(buffer, ble_data.buffer + ble_data.offset, size);
+	ble_data.offset += size;
+	return size;
+}
+
+static int receive_packet(dc_custom_io_t *io, suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
+{
+	if (io->packet_size < 64)
+		return receive_ble_packet(io, eon, buffer, size);
+	return receive_usbhid_packet(io, eon, buffer, size);
+}
+
+static int add_hdlc(unsigned char *dst, unsigned char val)
+{
+	int chars = 1;
+	switch (val) {
+	case 0x7e: case 0x7d:
+		*dst++ = 0x7d;
+		val ^= 0x20;
+		chars++;
+		/* fallthrough */
+	default:
+		*dst = val;
+	}
+	return chars;
+}
+
+static int hdlc_reencode(unsigned char *dst, unsigned char *src, int len)
+{
+	unsigned int crc = crc32(0, src, len);
+	int result = 0, i;
+
+	*dst++ = 0x7e; result++;
+	for (i = 0; i < len; i++) {
+		int chars = add_hdlc(dst, src[i]);
+		dst += chars;
+		result += chars;
+	}
+	for (i = 0; i < 4; i++) {
+		int chars = add_hdlc(dst, crc & 255);
+		dst += chars;
+		result += chars;
+		crc >>= 8;
+	}
+	*dst++ = 0x7e; result++;
+	return result;
+}
+
 static int send_cmd(suunto_eonsteel_device_t *eon,
 	unsigned short cmd,
 	unsigned int len,
@@ -169,6 +325,7 @@ static int send_cmd(suunto_eonsteel_device_t *eon,
 	unsigned char buf[64];
 	unsigned short seq = eon->seq;
 	unsigned int magic = eon->magic;
+	dc_custom_io_t *io = _dc_context_custom_io(eon->base.context);
 	dc_status_t rc = DC_STATUS_SUCCESS;
 	size_t transferred = 0;
 
@@ -200,7 +357,29 @@ static int send_cmd(suunto_eonsteel_device_t *eon,
 		memcpy(buf+14, buffer, len);
 	}
 
-	rc = dc_usbhid_write(eon->usbhid, buf, sizeof(buf), &transferred);
+	// BLE GATT protocol?
+	if (io->packet_size < 64) {
+		int hdlc_len;
+		unsigned char hdlc[2+2*(62+4)]; /* start/stop + escaping*(maxbuf+crc32) */
+		unsigned char *ptr;
+
+		hdlc_len = hdlc_reencode(hdlc, buf+2, buf[1]);
+
+		ptr = hdlc;
+		do {
+			int len = hdlc_len;
+
+			if (len > io->packet_size)
+				len = io->packet_size;
+			rc = io->packet_write(io, ptr, len, &transferred);
+			if (rc != DC_STATUS_SUCCESS)
+				break;
+			ptr += len;
+			hdlc_len -= len;
+		} while (hdlc_len);
+	} else {
+		rc = io->packet_write(io, buf, sizeof(buf), &transferred);
+	}
 	if (rc != DC_STATUS_SUCCESS) {
 		ERROR(eon->base.context, "write interrupt transfer failed");
 		return -1;
@@ -222,8 +401,11 @@ static int receive_header(suunto_eonsteel_device_t *eon, struct eon_hdr *hdr, un
 {
 	int ret;
 	unsigned char header[64];
+	dc_custom_io_t *io = _dc_context_custom_io(eon->base.context);
 
-	ret = receive_packet(eon, header, sizeof(header));
+	if (io->packet_size < 64)
+		fill_ble_data(io, eon);
+	ret = receive_packet(io, eon, header, sizeof(header));
 	if (ret < 0)
 		return -1;
 	if (ret < 12) {
@@ -249,11 +431,12 @@ static int receive_header(suunto_eonsteel_device_t *eon, struct eon_hdr *hdr, un
 static int receive_data(suunto_eonsteel_device_t *eon, unsigned char *buffer, int size)
 {
 	int ret = 0;
+	dc_custom_io_t *io = _dc_context_custom_io(eon->base.context);
 
 	while (size > 0) {
 		int len;
 
-		len = receive_packet(eon, buffer + ret, size);
+		len = receive_packet(io, eon, buffer + ret, size);
 		if (len < 0)
 			return -1;
 
@@ -348,7 +531,7 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
 		return -1;
 	}
 	memcpy(cmdbuf+4, filename, len);
-	rc = send_receive(eon, FILE_LOOKUP_CMD,
+	rc = send_receive(eon, CMD_FILE_OPEN,
 		len+4, cmdbuf,
 		sizeof(result), result);
 	if (rc < 0) {
@@ -357,7 +540,7 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
 	}
 	HEXDUMP (eon->base.context, DC_LOGLEVEL_DEBUG, "lookup", result, rc);
 
-	rc = send_receive(eon, FILE_STAT_CMD,
+	rc = send_receive(eon, CMD_FILE_STAT,
 		0, NULL,
 		sizeof(result), result);
 	if (rc < 0) {
@@ -377,7 +560,7 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
 			ask = 1024;
 		put_le32(1234, cmdbuf+0);	// Not file offset, after all
 		put_le32(ask, cmdbuf+4);	// Size of read
-		rc = send_receive(eon, FILE_READ_CMD,
+		rc = send_receive(eon, CMD_FILE_READ,
 			8, cmdbuf,
 			sizeof(result), result);
 		if (rc < 0) {
@@ -412,11 +595,11 @@ static int read_file(suunto_eonsteel_device_t *eon, const char *filename, dc_buf
 		size -= got;
 	}
 
-	rc = send_receive(eon, FILE_CLOSE_CMD,
+	rc = send_receive(eon, CMD_FILE_CLOSE,
 		0, NULL,
 		sizeof(result), result);
 	if (rc < 0) {
-		ERROR(eon->base.context, "cmd FILE_CLOSE_CMD failed");
+		ERROR(eon->base.context, "cmd CMD_FILE_CLOSE failed");
 		return -1;
 	}
 	HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "close", result, rc);
@@ -438,7 +621,7 @@ static struct directory_entry *parse_dirent(suunto_eonsteel_device_t *eon, int n
 		struct directory_entry *entry;
 
 		if (namelen + 8 + 1 > len || name[namelen] != 0) {
-			ERROR(eon->base.context, "corrupt dirent entry");
+			ERROR(eon->base.context, "corrupt dirent entry: len=%d namelen=%d name='%s'", len, namelen, name);
 			break;
 		}
 		HEXDUMP(eon->base.context, DC_LOGLEVEL_DEBUG, "dir entry", p, 8);
@@ -463,12 +646,11 @@ static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry *
 	unsigned char result[2048];
 	int rc, cmdlen;
 
-
 	*res = NULL;
 	put_le32(0, cmd);
 	memcpy(cmd + 4, dive_directory, sizeof(dive_directory));
 	cmdlen = 4 + sizeof(dive_directory);
-	rc = send_receive(eon, DIR_LOOKUP_CMD,
+	rc = send_receive(eon, CMD_DIR_OPEN,
 		cmdlen, cmd,
 		sizeof(result), result);
 	if (rc < 0) {
@@ -479,7 +661,7 @@ static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry *
 	for (;;) {
 		unsigned int nr, last;
 
-		rc = send_receive(eon, READDIR_CMD,
+		rc = send_receive(eon, CMD_DIR_READDIR,
 			0, NULL,
 			sizeof(result), result);
 		if (rc < 0) {
@@ -499,7 +681,7 @@ static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry *
 			break;
 	}
 
-	rc = send_receive(eon, DIR_CLOSE_CMD,
+	rc = send_receive(eon, CMD_DIR_CLOSE,
 		0, NULL,
 		sizeof(result), result);
 	if (rc < 0) {
@@ -513,26 +695,9 @@ static int get_file_list(suunto_eonsteel_device_t *eon, struct directory_entry *
 static int initialize_eonsteel(suunto_eonsteel_device_t *eon)
 {
 	const unsigned char init[] = {0x02, 0x00, 0x2a, 0x00};
-	unsigned char buf[64];
 	struct eon_hdr hdr;
 
-	dc_usbhid_set_timeout(eon->usbhid, 10);
-
-	/* Get rid of any pending stale input first */
-	/* NOTE! This will cause an annoying warning from dc_usbhid_read() */
-	for (;;) {
-		size_t transferred = 0;
-
-		dc_status_t rc = dc_usbhid_read(eon->usbhid, buf, sizeof(buf), &transferred);
-		if (rc != DC_STATUS_SUCCESS)
-			break;
-		if (!transferred)
-			break;
-	}
-
-	dc_usbhid_set_timeout(eon->usbhid, 5000);
-
-	if (send_cmd(eon, INIT_CMD, sizeof(init), init)) {
+	if (send_cmd(eon, CMD_INIT, sizeof(init), init)) {
 		ERROR(eon->base.context, "Failed to send initialization command");
 		return -1;
 	}
@@ -549,7 +714,7 @@ static int initialize_eonsteel(suunto_eonsteel_device_t *eon)
 }
 
 dc_status_t
-suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char *name, unsigned int model)
+suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char *name)
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	suunto_eonsteel_device_t *eon = NULL;
@@ -567,7 +732,12 @@ suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char
 	memset (eon->version, 0, sizeof (eon->version));
 	memset (eon->fingerprint, 0, sizeof (eon->fingerprint));
 
-	status = dc_usbhid_open(&eon->usbhid, context, 0x1493, 0x0030);
+	dc_custom_io_t *io = _dc_context_custom_io(eon->base.context);
+	if (io && io->packet_open)
+		status = io->packet_open(io, context, name);
+	else
+		status = dc_usbhid_custom_io(context, 0x1493, 0x0030);
+
 	if (status != DC_STATUS_SUCCESS) {
 		ERROR(context, "unable to open device");
 		goto error_free;
@@ -584,7 +754,7 @@ suunto_eonsteel_device_open(dc_device_t **out, dc_context_t *context, const char
 	return DC_STATUS_SUCCESS;
 
 error_close:
-	dc_usbhid_close(eon->usbhid);
+	suunto_eonsteel_device_close((dc_device_t *) eon);
 error_free:
 	free(eon);
 	return status;
@@ -703,12 +873,42 @@ suunto_eonsteel_device_foreach(dc_device_t *abstract, dc_dive_callback_t callbac
 	return device_is_cancelled(abstract) ? DC_STATUS_CANCELLED : DC_STATUS_SUCCESS;
 }
 
+static dc_status_t suunto_eonsteel_timesync(dc_device_t *abstract, const dc_datetime_t *datetime)
+{
+	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
+	unsigned char result[64], cmd[64];
+	int year, month, day;
+	int hour, min, sec, msec;
+	int rc;
+
+	year = datetime->year;
+	month = datetime->month;
+	day = datetime->day;
+	hour = datetime->hour;
+	min = datetime->minute;
+	sec = datetime->second;
+
+	INFO(eon->base.context, "SET_TIME: %d/%d/%d %d:%02d:%02d.%03d",
+		year, month, day, hour, min, sec, msec);
+
+	msec = sec * 1000;
+
+	cmd[0] = year & 255;
+	cmd[1] = year >> 8;
+	cmd[2] = month;
+	cmd[3] = day;
+	cmd[4] = hour;
+	cmd[5] = min;
+	cmd[6] = msec & 255;
+	cmd[7] = msec >> 8;
+
+	return send_receive(eon, CMD_SET_TIME, 8, cmd, sizeof(result), result);
+}
+
 static dc_status_t
 suunto_eonsteel_device_close(dc_device_t *abstract)
 {
-	suunto_eonsteel_device_t *eon = (suunto_eonsteel_device_t *) abstract;
+	dc_custom_io_t *io = _dc_context_custom_io(abstract->context);
 
-	dc_usbhid_close(eon->usbhid);
-
-	return DC_STATUS_SUCCESS;
+	return io->packet_close(io);
 }
