@@ -32,22 +32,11 @@
 #define vsnprintf _vsnprintf
 #endif
 
-#include <libdivecomputer/suunto_eonsteel.h>
-
+#include "suunto_eonsteel.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
-
-#ifdef _MSC_VER
-#define strcasecmp _stricmp
-#if _MSC_VER < 1800
-// The rint() function is only available in MSVC 2013 and later
-// versions. Our replacement macro isn't entirely correct, because the
-// rounding rules for halfway cases are slightly different (away from
-// zero vs to even). But for our use-case, that's not a problem.
-#define rint(x) ((x) >= 0.0 ? floor((x) + 0.5): ceil((x) - 0.5))
-#endif
-#endif
+#include "platform.h"
 
 #define C_ARRAY_SIZE(a) (sizeof(a) / sizeof(*(a)))
 
@@ -1072,7 +1061,11 @@ suunto_eonsteel_parser_get_datetime(dc_parser_t *parser, dc_datetime_t *datetime
 	if (parser->size < 4)
 		return DC_STATUS_UNSUPPORTED;
 
-	dc_datetime_gmtime(datetime, array_uint32_le(parser->data));
+	if (!dc_datetime_gmtime(datetime, array_uint32_le(parser->data)))
+		return DC_STATUS_DATAFORMAT;
+
+	datetime->timezone = DC_TIMEZONE_NONE;
+
 	return DC_STATUS_SUCCESS;
 }
 
